#coding=utf-8

import pyaudio
import numpy as np
import math
from . import cache
fps = 4096*10*8
p = pyaudio.PyAudio()
nbyte = 2
ndtype = np.int16
nmax = (1<<(nbyte<<3))-1

nrange = nmax/2
nrange = (nmax+1)/2
def fn(n):
    """
        乐谱音符转频率
        C1: n=4 fn(4)=32.703
        C1#: n=5
        D1: n=6
    """
    return 27.5*(2**(1/12))**(n-1)

pass

def f1(rate, n, sec = 1.0, msound = 1.0):
    """
    rate: 频率
    n: 乐谱音符（1234567）
    sec: 声波发音时间，单位秒
    msound: 全局音量大小
    """
    global fps #每秒取样数
    size = int(fps * sec)
    prev = 0#int(size*0.02)
    data = np.zeros(prev+size+int(size*0.7), dtype = np.float32)
    # 频率越高，声音越小
    #sound = 1-(mid-n)*(mid-n)/(120*120)
    sound = (100-n)**2/100**2
    #if n > mid:
    #    sound = -sound
    offset = random.random()*math.pi
    for i in range(len(data)):
        s0 = i/fps
        w1 = (s0**0.25)
        w2 = math.exp(-1*(s0**2)/(10*(0.2**2)))
        y = w1*math.sin(offset+2*math.pi*s0*rate)*w2
        #y = math.exp(-1/(2*0.2*0.2)*s0*s0)
        #y=math.exp(-1*(s0**2)/(11*(0.2**2)))
        #y=w1*w2
        y*=(400/rate)**0.5
        y*=0.5
        #y*=nrange
        y*=msound
        data[prev+i] = y
    return data#rst

pass
mid = 50
import random
def f2(rate, n, sec = 1.0, msound = 1.0, index=0):
    """
    rate: 频率
    n: 乐谱音符（1234567）
    sec: 声波发音时间，单位秒
    msound: 全局音量大小
    """
    global fps #每秒取样数
    size = int(fps * sec)
    prev = 0#int(size*0.02)
    data = np.zeros(prev+size+int(size*0.5), dtype = np.float32)
    # 频率越高，声音越小
    #sound = 1-(mid-n)*(mid-n)/(120*120)
    sound = (100-n)**2/100**2
    #if n > mid:
    #    sound = -sound
    offset = index*math.pi*0.03*random.random()*0
    #print(size)
    for i in range(size):
        #s0 = rate*i
        #y = (s0**0.25)*math.sin(2*math.pi*s0/fps)*math.exp(-1/(2*0.2*0.2)*s0*s0)
        x0 = i*2*math.pi*rate/fps
        #x0 += (random.random()-0.5)*0.1
        #sound = math.cos(abs(mid-n)/50*math.pi*0.5)
        y = math.sin(x0+offset)*(0.1*sound+sound*0.9)
        #y *=(1+(random.random()-0.5)*0.1)
        # 音量从1到0
        r = (size-i)/size
        y *= r*msound
        #y *= math.cos(i*math.pi/size)*0.5+0.5
        #y *= nrange
        y = max(-nrange+1,min(nrange-1, y))
        data[prev+i] = y
    return data#rst

pass
f=f2
import random
class CacheFc:
    def __init__(self,n=10):
        #self.cache = {}
        self.index=0
        self.n = n
        self.caches = []
        for i in range(n):
            self.caches.append({})
    def __call__(self, rate, n):
        i = self.index
        self.index=(self.index+1)%self.n
        cache = self.caches[0]
        if rate not in cache:
            cache[rate] = f(rate, n, 1.0)
        return cache[rate]

pass
def create():
    return p.open(format=p.get_format_from_width(nbyte), channels=1, rate=fps, output=True)

pass

def close(stream):
    stream.stop_stream()
    stream.close()

pass
import threading


class Sound:
    def __init__(self):
        self.fc = CacheFc()
        self.zero = create()
        #print(fps)
        self.stream = cache.Stream(self.zero.write, nrange,100, int(fps*0.05), ndtype, left=1)
        self.stream.start()
        # self.frees = [create() for i in range(20)]
        # self.zero = create() #一直播放着空旋律，避免噪音
        # self.index = 0
        # self.lock = threading.Lock()
        # self.running = True
        # th = threading.Thread(target = self.run_zero)
        # th.setDaemon(True)
        # th.start()
    def play(self, n):
        if not self.stream.running:
            return
        rate = fn(n)
        data = self.fc(rate, n)
        self.stream.add(data)
    def close(self):
        import time
        self.stream.stop()
        time.sleep(1.0)
        close(self.zero)

pass
sd = Sound()
def release():
    p.terminate()

pass



#coding=utf-8

import pynput
from pynput.keyboard import Listener, Key
from biano import sound1 as sound


ctls = list("1234567890")
c2base = {}
c2i = {}
for c in ctls[:len(ctls)>>1]:
    c2base[c] = 0
    c2i[c] = int(c)-1

pass

for c in ctls[len(ctls)>>1:]:
    c2base[c] = 1
    c2i[c] = int(c)-6
    if c2i[c] <0:
        c2i[c] = 4

pass

lefts = "qwertasdfzxc"
rights = "yuiophjklnm,"
nlefts = {k:i for k,i in zip(lefts, range(len(lefts)))}
nrights = {k:i for k,i in zip(rights, range(len(rights)))}
maps = {}
k2base = {}
for k in nlefts:
    maps[k] = nlefts[k]
    k2base[k] = 0
for k in nrights:
    maps[k] = nrights[k]
    k2base[k] = 1

pass
bases = [40,40]
moves = [0,0]
mvs = "-="
pressed = set()
pressed = set()
c='3'
bases[c2base[c]] = c2i[c]*len(lefts)+16
c='9'
bases[c2base[c]] = c2i[c]*len(lefts)+16
def press(key):
    global moves
    if hasattr(key, "char"):
        c = key.char
        if c in maps:
            if c in pressed:
                return
            vc = maps[c] + bases[k2base[c]]+moves[k2base[c]]
            if vc <= 0:
                return
            if vc >= 89:
                return
            sound.sd.play(vc)
            #print(vc)
            pressed.add(c)
            #print(vc)
        elif c in mvs:
            if c == "-":
                moves = [0,0]
            else:
                moves = [-24, 24]
        elif c in ctls:
            bases[c2base[c]] = c2i[c]*len(lefts)+16
        # elif c in maps:
        #     if c in pressed:
        #         return
        #     vc = maps[c] + bases[k2base[c]]+moves[k2base[c]]
        #     if vc <= 0:
        #         return
        #     if vc >= 89:
        #         return
        #     pressed.add(c)
        #     #print(vc)
        #     sound.sd.play(vc)
    global lst
    if key == Key.esc:
        lst.stop()
        sound.sd.close()
        sound.release()

pass

def release(key):
    if hasattr(key, "char"):
        c = key.char
        if c not in maps:
            return
        if c in pressed:
            pressed.remove(c)

pass
lst = None
def run():
    global lst
    print("按键: 左手: qwertasdfzxc 右手: yuiophjklnm,")
    print("修改音调: 左手按键: 1,2,3,4,5 右手按键: 6,7,8,9,0")
    print("修改模式: 按-或=")
    print("按esc退出")
    print("press to sound: left: qwertasdfzxc right: yuiophjklnm,")
    print("change tone: left: press 1,2,3,4,5 right: press 6,7,8,9,0")
    print("change mode: press - or =")
    print("press esc to exit")
    with Listener(on_press=press, on_release=release) as lst:
        lst.join()

pass
if __name__=="__main__":
    run()

pass
"""
python keys.py
"""


#coding=utf-8

import pyaudio
import numpy as np
import math
fps = 4096*10*2
p = pyaudio.PyAudio()
nbyte = 2
ndtype = np.int16
nmax = (1<<(nbyte<<3))-1

nrange = nmax/2
nrange = (nmax+1)/2
def fn(n):
    """
        乐谱音符转频率
        C1: n=4 fn(4)=32.703
        C1#: n=5
        D1: n=6
    """
    return 27.5*(2**(1/12))**(n-1)

pass

def f1(rate, n, sec = 1.0, msound = 1.0):
    """
    rate: 频率
    n: 乐谱音符（1234567）
    sec: 声波发音时间，单位秒
    msound: 全局音量大小
    """
    global fps #每秒取样数
    size = int(fps * sec)
    prev = 0#int(size*0.02)
    data = np.zeros(prev+size+int(size+0.5), dtype = ndtype)
    # 频率越高，声音越小
    #sound = 1-(mid-n)*(mid-n)/(120*120)
    sound = (100-n)**2/100**2
    #if n > mid:
    #    sound = -sound
    offset = random.random()*math.pi
    for i in range(size):
        s0 = i/fps
        y = (s0**0.25)*math.sin(offset+2*math.pi*s0*rate)*math.exp(-1/(2*0.2*0.2)*s0*s0)
        #y = math.exp(-1/(2*0.2*0.2)*s0*s0)
        y*=(400/rate)**0.5
        y*=0.5
        y*=nrange
        y*=msound
        data[prev+i] = y
    return data#rst

pass
mid = 50
import random
def f(rate, n, sec = 1.0, msound = 1.0):
    """
    rate: 频率
    n: 乐谱音符（1234567）
    sec: 声波发音时间，单位秒
    msound: 全局音量大小
    """
    global fps #每秒取样数
    size = int(fps * sec)
    prev = 0#int(size*0.02)
    data = np.zeros(prev+size+int(size+0.5), dtype = ndtype)
    if msound==0.001:
        print("zero")
        data[:] = 0
        return data
    # 频率越高，声音越小
    #sound = 1-(mid-n)*(mid-n)/(120*120)
    sound = (100-n)**2/100**2
    #if n > mid:
    #    sound = -sound
    for i in range(size):
        #s0 = rate*i
        #y = (s0**0.25)*math.sin(2*math.pi*s0/fps)*math.exp(-1/(2*0.2*0.2)*s0*s0)
        x0 = i*2*math.pi*rate/fps
        #x0 += (random.random()-0.5)*0.1
        #sound = math.cos(abs(mid-n)/50*math.pi*0.5)
        y = math.sin(x0)*(0.1*sound+sound*0.9)
        #y *=(1+(random.random()-0.5)*0.1)
        # 音量从1到0
        r = (size-i)/size
        y *= r*msound
        #y *= math.cos(i*math.pi/size)*0.5+0.5
        y *= nrange
        y = max(-nrange+1,min(nrange-1, y))
        data[prev+i] = y
    return data#rst

pass

class CacheFc:
    def __init__(self):
        self.cache = {}
    def __call__(self, rate, n):
        if rate not in self.cache:
            self.cache[rate] = f(rate, n, 1.0)
        return self.cache[rate]

pass
def create():
    return p.open(format=p.get_format_from_width(nbyte), channels=1, rate=fps, output=True)

pass

def close(stream):
    stream.stop_stream()
    stream.close()

pass
import threading


class Sound:
    def run_zero(self):
        global fps
        size = int(fps)
        n = 50
        rate = fn(n)
        data = f(rate, n, 2.0, 0.001)
        while self.running:
            self.zero.write(data.tobytes())
        close(self.zero)
    def __init__(self):
        self.fc = CacheFc()
        self.frees = [create() for i in range(20)]
        self.zero = create() #一直播放着空旋律，避免噪音
        self.index = 0
        self.lock = threading.Lock()
        self.running = True
        th = threading.Thread(target = self.run_zero)
        th.setDaemon(True)
        th.start()
    def _play(self, data, id):
        self.frees[id].write(data.tobytes())
    def play(self, n):
        if not self.running:
            return
        rate = fn(n)
        data = self.fc(rate, n)
        with self.lock:
            id = self.index
            self.index = (self.index+1)%20
        th = threading.Thread(target = self._play, args =(data, id))
        th.setDaemon(True)
        th.start()
    def close(self):
        import time
        self.running = False
        time.sleep(1.0)
        for st in self.frees:
            close(st)

pass
sd = Sound()
def release():
    p.terminate()

pass



#coding=utf-8

from matplotlib import pyplot as plt

import numpy as np
import math
from buildz import pyz
pyz.add(__file__)
from biano import sound1 as sound
# mn,mx=None,None
# for i in range(0, 100):
#     y = sound.f(sound.fn(i), i)
#     if mn is None:
#         mn = min(y)
#     else:
#         mn = min(min(y), mn)
#     if mx is None:
#         mx = max(y)
#     else:
#         mx = max(max(y), mx)
#     print(max(y), min(y))

# pass
# print(mx, mn)
# exit()
i = 39
y = sound.f(sound.fn(i), i)
#y=y[:len(y)>>1]
x = np.array(list(range(0, len(y))),dtype=np.float32)
x = x*(1/sound.fps)#/len(x))
print(max(y), min(y))
plt.plot(x,y)
plt.show()


#!/usr/bin/env python
# -*- coding:utf-8 -*-
# Author: Zzz(1309458652@qq.com)
# Description:

from setuptools import setup, find_packages

setup(
    name = 'biano',
    version = '0.2.2',
    keywords='biano',
    long_description=open('README.md', 'r', encoding="utf-8").read(),
    long_description_content_type='text/markdown',
    description = "keyboard piano, just start",
    license = 'Apache License 2.0',
    url = 'https://github.com/buildCodeZ/biano',
    author = 'Zzz',
    author_email = '1309458652@qq.com',
    packages = find_packages(),
    include_package_data = True,
    platforms = 'any',
    install_requires = ['pynput>=1.7.6', 'numpy>=1.22.2', 'PyAudio>=0.2.12'],
)


#coding=utf-8

import sys
from buildz import pyz
import numpy as np
import threading as th
import time
sys.path.append(r"D:\downloads\codes\python\pydub-0.25.1")
pyz.add(__file__)

from pydub import AudioSegment
audio_file=r"D:\rootz\data\music\稻香guita.mp3"
audio = AudioSegment.from_file(audio_file)
x = audio.get_array_of_samples()
nx = np.array(x)

from biano import sound
fps = len(nx)/audio.duration_seconds
import threading as th
fps = int(fps)
nbyte = 2
def write(dt):
    lk = th.Lock()
    #s.write(dt.tobytes())
    #return
    n=100
    sz = len(nx)//n
    for i in range(n):
        with lk:
            dt = nx[i*sz:i*sz+sz]
            dt = dt*0.1
            dt=dt*1.0
            dt = dt.astype(sound.ndtype)
            s.write(nx[i*sz:i*sz+sz].tobytes())
            print(i*sz, i*sz+sz)
    #s.write(nx)

pass
s = sound.p.open(format=sound.p.get_format_from_width(nbyte), channels=1, rate=fps, output=True)
t = th.Thread(target=write, args = (nx,),daemon=True)
t.start()
#s.write(nx)
try:
    while t.is_alive():
        time.sleep(1.0)
finally:
    sound.close(s)

pass



#coding=utf-8
import pyaudio
import numpy as np
import math
from buildz.tools import *
import threading as th
import time
class Stream(Base):
    def run(self):
        self.running = True
        self.loop()
    def start(self):
        t = th.Thread(target=self.run,daemon=True)
        t.start()
        self.t = t
    def stop(self):
        self.running=False
    def init(self, output, nrange ,num, size, ndtype, left=1, r_new = 1.0, r_old = 0.7):
        self.r_new = r_new
        self.ndtype = ndtype
        self.r_old = r_old
        self.win = size
        self.range = nrange
        self.num = num
        self.left = left
        self.size = size
        self.total = size*num
        self.data = np.zeros(size*num, dtype = np.float32)
        self.out_data = None
        self.lock = th.Lock()
        self.running = False
        self.output = output
        self.offset = 0
        self.lock_offset = -1
    def loop(self):
        while self.running:
            while self.running:
                with self.lock:
                    if self.offset==self.lock_offset:
                        time.sleep(0.001)
                        continue
                    dt = self.data[self.offset*self.size:self.offset*self.size+self.size]
                    tmp=(dt*self.range).astype(self.ndtype)
                    dt[:]=0
                    self.offset=(self.offset+1)%self.num
                    break
            self.output(tmp.tobytes())
    def add(self, data):
        with self.lock:
            off = (self.offset+self.left)%self.num
            self.lock_offset = off
        l = data.size
        l1 = min((self.num-off)*self.size, l)
        tmp = self.data[off*self.size:off*self.size+l1]
        data1=data
        if l1!=l:
            data1=data[:l1]
        tmp[:] = np.maximum(np.minimum(1*(tmp*self.r_old+data1*self.r_new),1.0), -1.0)
        #tmp[:] = np.maximum(np.minimum(tmp, 1.0), -1.0)
        if l1!=l:
            left = l-l1
            tmp = self.data[:left]
            data1 = data[l1:]
            tmp[:] = np.maximum(np.minimum(1*(tmp*self.r_old+data1*self.r_new),1.0), -1.0)
            #tmp[:] = np.maximum(np.minimum(tmp, 1.0), -1.0)
        with self.lock:
            self.lock_offset=-1

pass

